Struct bones_framework::prelude::piccolo::registry::Registry

source ·
pub struct Registry<'gc> { /* private fields */ }

Implementations

source

impl<'gc> Registry<'gc>

source

pub fn new(mc: &Mutation<'gc>) -> Registry<'gc>

source

pub fn roots(&self) -> DynamicRootSet<'gc>

source

pub fn singleton<S>(&self, ctx: Context<'gc>) -> &'gc <S as Rootable<'gc>>::Root
where S: for<'a> Rootable<'a>, <S as Rootable<'gc>>::Root: Singleton<'gc>,

Create an instance of a type that exists at most once per Lua instance.

If the type has already been created, returns the already created instance, otherwise calls S::create to create a new instance and returns it.

source

pub fn stash<S>( &self, mc: &Mutation<'gc>, s: S, ) -> <S as Stashable<'gc>>::Stashed
where S: Stashable<'gc>,

“Stash” a value with a 'gc branding lifetime, creating a 'static handle to it.

This is a wrapper around an internal gc-arena::DynamicRootSet that makes it a little simpler to work with common piccolo types without having to manually specify Rootable projections.

It can be implemented for external types by implementing the Stashable trait.

source

pub fn fetch<F>(&self, f: &F) -> <F as Fetchable<'gc>>::Fetched
where F: Fetchable<'gc>,

“Fetch” the real value for a handle that has been returned from Registry::stash.

It can be implemented for external types by implementing the Fetchable trait.